import { BaseMetadata, FileMetadata, ResourceProcessor } from "../types";
import { Resource, db, ResourceProcessStatus } from "@nicestack/common";
import { extname } from "path";
import mime from "mime";
import { calculateFileHash, getUploadFilePath } from "@server/utils/file";
import { Logger } from "@nestjs/common";
import { statSync } from "fs";  // Add this import
export class GeneralProcessor implements ResourceProcessor {
    private logger = new Logger(GeneralProcessor.name);

    async process(resource: Resource): Promise<Resource> {
        const originMeta = resource.metadata as any;
        try {
            // 提取文件扩展名作为type
            const fileExtension = extname(resource.filename).toLowerCase().slice(1);
            this.logger.debug(`文件扩展名: ${fileExtension}`);

            // 获取文件路径并验证
            const filePath = getUploadFilePath(resource.fileId);
            this.logger.debug(`文件路径: ${filePath}`);
            const fileStats = statSync(filePath);
            const fileSize = fileStats.size;
            const modifiedAt = fileStats.mtime;
            const createdAt = fileStats.birthtime;

            const fileHash = await calculateFileHash(filePath);
            // 准备metadata
            const metadata: BaseMetadata = {
                filename: resource.filename,
                extension: fileExtension,
                mimeType: mime.lookup(fileExtension) || 'application/octet-stream',
                size: fileSize,
                modifiedAt,
                createdAt,
            };

            const updatedResource = await db.resource.update({
                where: { id: resource.id },
                data: {
                    type: fileExtension,
                    hash: fileHash,
                    metadata: { ...originMeta, ...metadata } as any,
                }
            });

            return updatedResource;

        } catch (error: any) {
            throw new Error(`Resource processing failed: ${error.message}`);
        }
    }
}